/*
 * 
 */
package de.bsvrz.sys.startstopp.startstopp;

import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.authentication.ClientCredentials;
import de.bsvrz.dav.daf.main.authentication.UserProperties;

public class ZugangDav {

    private final String userName;
    private final String adresse;
    private final int port;
    private final UserProperties userProperties;

    public ZugangDav(String[] args) throws MissingParameterException {
        
        ClientDavParameters davParameter = new ClientDavParameters(args);
        userName = davParameter.getUserName();
        if (userName == null || userName.isEmpty()) {
            throw new MissingParameterException(
                    "StartStopp benötigt einen Benutzername - Parameter: '-benutzer= {name}'");
        }

        adresse = davParameter.getDavCommunicationAddress();
        if (adresse == null || adresse.isEmpty()) {
            throw new MissingParameterException(
                    "StartStopp benötigt den Hostname für eine Datenverteilerverbindung - Parameter: '-datenverteiler= {host:port}'");
        }

        port = davParameter.getDavCommunicationSubAddress();
        if (port <= 0) {
            throw new MissingParameterException(
                    "StartStopp benötigt den Port für eine Datenverteilerverbindung - Parameter: '-datenverteiler= {host:port}'");
        }

        userProperties = davParameter.getUserProperties();
        if (userProperties == null) {
            throw new MissingParameterException(
                    "StartStopp benötigt die Angabe der Authentifizierungsdatei für eine Datenverteilerverbindung - Parameter: '-authentifizierung= {pfad}'");
        }
    }

    public ClientCredentials getCredencials() {
        return userProperties.getClientCredentials(userName);
    }

    public String getUserName() {
        return userName;
    }

    public String getAdresse() {
        return adresse;
    }

    public int getPort() {
        return port;
    }
}
